@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2002, 2004, 2006, 2007, 2008, 2009, 2010, 2011 Free
- * Software Foundation, Inc.
+ * Copyright (C) 2002-2012 Free Software Foundation, Inc.
  *
  * This file is part of LIBTASN1.
  *
@@ -38,19 +37,19 @@ extern char _asn1_identifierMissing[];
 
 
 /******************************************************/
-/* Function : _asn1_add_node_only                     */
+/* Function : _asn1_add_single_node                     */
 /* Description: creates a new NODE_ASN element.       */
 /* Parameters:                                        */
-/*   type: type of the new element (see TYPE_         */
+/*   type: type of the new element (see ASN1_ETYPE_         */
 /*         and CONST_ constants).                     */
 /* Return: pointer to the new element.                */
 /******************************************************/
-ASN1_TYPE
-_asn1_add_node_only (unsigned int type)
+asn1_node
+_asn1_add_single_node (unsigned int type)
 {
-  ASN1_TYPE punt;
+  asn1_node punt;
 
-  punt = (ASN1_TYPE) _asn1_calloc (1, sizeof (struct node_asn_struct));
+  punt = calloc (1, sizeof (struct asn1_node_st));
   if (punt == NULL)
     return NULL;
 
@@ -68,8 +67,8 @@ _asn1_add_node_only (unsigned int type)
 /*   node: NODE_ASN element pointer.                              */
 /* Return: NULL if not found.                                     */
 /******************************************************************/
-ASN1_TYPE
-_asn1_find_left (ASN1_TYPE node)
+asn1_node
+_asn1_find_left (asn1_node node)
 {
   if ((node == NULL) || (node->left == NULL) || (node->left->down == node))
     return NULL;
@@ -78,12 +77,12 @@ _asn1_find_left (ASN1_TYPE node)
 }
 
 
-asn1_retCode
-_asn1_create_static_structure (ASN1_TYPE pointer, char *output_file_name,
+int
+_asn1_create_static_structure (asn1_node pointer, char *output_file_name,
 			       char *vector_name)
 {
   FILE *file;
-  ASN1_TYPE p;
+  asn1_node p;
   unsigned long t;
 
   file = fopen (output_file_name, "w");
@@ -97,7 +96,7 @@ _asn1_create_static_structure (ASN1_TYPE pointer, char *output_file_name,
 
   fprintf (file, "#include <libtasn1.h>\n\n");
 
-  fprintf (file, "const ASN1_ARRAY_TYPE %s[] = {\n", vector_name);
+  fprintf (file, "const asn1_static_node %s[] = {\n", vector_name);
 
   p = pointer;
 
@@ -105,7 +104,7 @@ _asn1_create_static_structure (ASN1_TYPE pointer, char *output_file_name,
     {
       fprintf (file, "  { ");
 
-      if (p->name)
+      if (p->name[0] != 0)
 	fprintf (file, "\"%s\", ", p->name);
       else
 	fprintf (file, "NULL, ");
@@ -169,22 +168,23 @@ _asn1_create_static_structure (ASN1_TYPE pointer, char *output_file_name,
  * @array is a vector created by asn1_parser2array().
  *
  * Returns: %ASN1_SUCCESS if structure was created correctly,
- *   %ASN1_ELEMENT_NOT_EMPTY if *@definitions not ASN1_TYPE_EMPTY,
+ *   %ASN1_ELEMENT_NOT_EMPTY if *@definitions not NULL,
  *   %ASN1_IDENTIFIER_NOT_FOUND if in the file there is an identifier
  *   that is not defined (see @errorDescription for more information),
  *   %ASN1_ARRAY_ERROR if the array pointed by @array is wrong.
  **/
-asn1_retCode
-asn1_array2tree (const ASN1_ARRAY_TYPE * array, ASN1_TYPE * definitions,
+int
+asn1_array2tree (const asn1_static_node * array, asn1_node * definitions,
 		 char *errorDescription)
 {
-  ASN1_TYPE p, p_last = NULL;
+  asn1_node p, p_last = NULL;
   unsigned long k;
   int move;
-  asn1_retCode result;
+  int result;
+  unsigned int type;
 
 
-  if (*definitions != ASN1_TYPE_EMPTY)
+  if (*definitions != NULL)
     return ASN1_ELEMENT_NOT_EMPTY;
 
   move = UP;
@@ -192,7 +192,9 @@ asn1_array2tree (const ASN1_ARRAY_TYPE * array, ASN1_TYPE * definitions,
   k = 0;
   while (array[k].value || array[k].type || array[k].name)
     {
-      p = _asn1_add_node (array[k].type & (~CONST_DOWN));
+      type = convert_old_type(array[k].type);
+
+      p = _asn1_add_static_node (type & (~CONST_DOWN));
       if (array[k].name)
 	_asn1_set_name (p, array[k].name);
       if (array[k].value)
@@ -208,9 +210,9 @@ asn1_array2tree (const ASN1_ARRAY_TYPE * array, ASN1_TYPE * definitions,
 
       p_last = p;
 
-      if (array[k].type & CONST_DOWN)
+      if (type & CONST_DOWN)
 	move = DOWN;
-      else if (array[k].type & CONST_RIGHT)
+      else if (type & CONST_RIGHT)
 	move = RIGHT;
       else
 	{
@@ -264,7 +266,7 @@ asn1_array2tree (const ASN1_ARRAY_TYPE * array, ASN1_TYPE * definitions,
   if (result != ASN1_SUCCESS)
     {
       _asn1_delete_list_and_nodes ();
-      *definitions = ASN1_TYPE_EMPTY;
+      *definitions = NULL;
     }
   else
     _asn1_delete_list ();
@@ -277,17 +279,17 @@ asn1_array2tree (const ASN1_ARRAY_TYPE * array, ASN1_TYPE * definitions,
  * @structure: pointer to the structure that you want to delete.
  *
  * Deletes the structure *@structure.  At the end, *@structure is set
- * to ASN1_TYPE_EMPTY.
+ * to NULL.
  *
  * Returns: %ASN1_SUCCESS if successful, %ASN1_ELEMENT_NOT_FOUND if
- *   *@structure was ASN1_TYPE_EMPTY.
+ *   *@structure was NULL.
  **/
-asn1_retCode
-asn1_delete_structure (ASN1_TYPE * structure)
+int
+asn1_delete_structure (asn1_node * structure)
 {
-  ASN1_TYPE p, p2, p3;
+  asn1_node p, p2, p3;
 
-  if (*structure == ASN1_TYPE_EMPTY)
+  if (*structure == NULL)
     return ASN1_ELEMENT_NOT_FOUND;
 
   p = *structure;
@@ -329,7 +331,7 @@ asn1_delete_structure (ASN1_TYPE * structure)
 	}
     }
 
-  *structure = ASN1_TYPE_EMPTY;
+  *structure = NULL;
   return ASN1_SUCCESS;
 }
 
@@ -346,14 +348,14 @@ asn1_delete_structure (ASN1_TYPE * structure)
  * Returns: %ASN1_SUCCESS if successful, %ASN1_ELEMENT_NOT_FOUND if
  *   the @element_name was not found.
  **/
-asn1_retCode
-asn1_delete_element (ASN1_TYPE structure, const char *element_name)
+int
+asn1_delete_element (asn1_node structure, const char *element_name)
 {
-  ASN1_TYPE p2, p3, source_node;
+  asn1_node p2, p3, source_node;
 
   source_node = asn1_find_node (structure, element_name);
 
-  if (source_node == ASN1_TYPE_EMPTY)
+  if (source_node == NULL)
     return ASN1_ELEMENT_NOT_FOUND;
 
   p2 = source_node->right;
@@ -372,16 +374,16 @@ asn1_delete_element (ASN1_TYPE structure, const char *element_name)
   return asn1_delete_structure (&source_node);
 }
 
-ASN1_TYPE
-_asn1_copy_structure3 (ASN1_TYPE source_node)
+asn1_node
+_asn1_copy_structure3 (asn1_node source_node)
 {
-  ASN1_TYPE dest_node, p_s, p_d, p_d_prev;
+  asn1_node dest_node, p_s, p_d, p_d_prev;
   int move;
 
   if (source_node == NULL)
     return NULL;
 
-  dest_node = _asn1_add_node_only (source_node->type);
+  dest_node = _asn1_add_single_node (source_node->type);
 
   p_s = source_node;
   p_d = dest_node;
@@ -392,48 +394,37 @@ _asn1_copy_structure3 (ASN1_TYPE source_node)
     {
       if (move != UP)
 	{
-	  if (p_s->name)
-	    _asn1_set_name (p_d, p_s->name);
+	  if (p_s->name[0] != 0)
+	    _asn1_cpy_name (p_d, p_s);
 	  if (p_s->value)
 	    _asn1_set_value (p_d, p_s->value, p_s->value_len);
-	  move = DOWN;
-	}
-      else
-	move = RIGHT;
-
-      if (move == DOWN)
-	{
 	  if (p_s->down)
 	    {
 	      p_s = p_s->down;
 	      p_d_prev = p_d;
-	      p_d = _asn1_add_node_only (p_s->type);
+	      p_d = _asn1_add_single_node (p_s->type);
 	      _asn1_set_down (p_d_prev, p_d);
+	      continue;
 	    }
-	  else
-	    move = RIGHT;
 	}
 
       if (p_s == source_node)
-	break;
+        break;
 
-      if (move == RIGHT)
-	{
-	  if (p_s->right)
-	    {
-	      p_s = p_s->right;
-	      p_d_prev = p_d;
-	      p_d = _asn1_add_node_only (p_s->type);
-	      _asn1_set_right (p_d_prev, p_d);
-	    }
-	  else
+      if (p_s->right)
+        {
+	    move = RIGHT;
+	    p_s = p_s->right;
+	    p_d_prev = p_d;
+	    p_d = _asn1_add_single_node (p_s->type);
+	    _asn1_set_right (p_d_prev, p_d);
+        }
+      else
+        {
 	    move = UP;
-	}
-      if (move == UP)
-	{
-	  p_s = _asn1_find_up (p_s);
-	  p_d = _asn1_find_up (p_d);
-	}
+	    p_s = _asn1_find_up (p_s);
+	    p_d = _asn1_find_up (p_d);
+        }
     }
   while (p_s != source_node);
 
@@ -441,10 +432,10 @@ _asn1_copy_structure3 (ASN1_TYPE source_node)
 }
 
 
-static ASN1_TYPE
-_asn1_copy_structure2 (ASN1_TYPE root, const char *source_name)
+static asn1_node
+_asn1_copy_structure2 (asn1_node root, const char *source_name)
 {
-  ASN1_TYPE source_node;
+  asn1_node source_node;
 
   source_node = asn1_find_node (root, source_name);
 
@@ -453,10 +444,10 @@ _asn1_copy_structure2 (ASN1_TYPE root, const char *source_name)
 }
 
 
-static asn1_retCode
-_asn1_type_choice_config (ASN1_TYPE node)
+static int
+_asn1_type_choice_config (asn1_node node)
 {
-  ASN1_TYPE p, p2, p3, p4;
+  asn1_node p, p2, p3, p4;
   int move, tlen;
 
   if (node == NULL)
@@ -469,20 +460,20 @@ _asn1_type_choice_config (ASN1_TYPE node)
     {
       if (move != UP)
 	{
-	  if ((type_field (p->type) == TYPE_CHOICE) && (p->type & CONST_TAG))
+	  if ((type_field (p->type) == ASN1_ETYPE_CHOICE) && (p->type & CONST_TAG))
 	    {
 	      p2 = p->down;
 	      while (p2)
 		{
-		  if (type_field (p2->type) != TYPE_TAG)
+		  if (type_field (p2->type) != ASN1_ETYPE_TAG)
 		    {
 		      p2->type |= CONST_TAG;
 		      p3 = _asn1_find_left (p2);
 		      while (p3)
 			{
-			  if (type_field (p3->type) == TYPE_TAG)
+			  if (type_field (p3->type) == ASN1_ETYPE_TAG)
 			    {
-			      p4 = _asn1_add_node_only (p3->type);
+			      p4 = _asn1_add_single_node (p3->type);
 			      tlen = _asn1_strlen (p3->value);
 			      if (tlen > 0)
 				_asn1_set_value (p4, p3->value, tlen + 1);
@@ -499,7 +490,7 @@ _asn1_type_choice_config (ASN1_TYPE node)
 	      while (p2)
 		{
 		  p3 = p2->right;
-		  if (type_field (p2->type) == TYPE_TAG)
+		  if (type_field (p2->type) == ASN1_ETYPE_TAG)
 		    asn1_delete_structure (&p2);
 		  p2 = p3;
 		}
@@ -538,10 +529,10 @@ _asn1_type_choice_config (ASN1_TYPE node)
 }
 
 
-static asn1_retCode
-_asn1_expand_identifier (ASN1_TYPE * node, ASN1_TYPE root)
+static int
+_asn1_expand_identifier (asn1_node * node, asn1_node root)
 {
-  ASN1_TYPE p, p2, p3;
+  asn1_node p, p2, p3;
   char name2[ASN1_MAX_NAME_SIZE + 2];
   int move;
 
@@ -555,17 +546,15 @@ _asn1_expand_identifier (ASN1_TYPE * node, ASN1_TYPE root)
     {
       if (move != UP)
 	{
-	  if (type_field (p->type) == TYPE_IDENTIFIER)
+	  if (type_field (p->type) == ASN1_ETYPE_IDENTIFIER)
 	    {
-	      _asn1_str_cpy (name2, sizeof (name2), root->name);
-	      _asn1_str_cat (name2, sizeof (name2), ".");
-	      _asn1_str_cat (name2, sizeof (name2), (char *) p->value);
+	      snprintf(name2, sizeof (name2), "%s.%s", root->name, p->value);
 	      p2 = _asn1_copy_structure2 (root, name2);
 	      if (p2 == NULL)
 		{
 		  return ASN1_IDENTIFIER_NOT_FOUND;
 		}
-	      _asn1_set_name (p2, p->name);
+	      _asn1_cpy_name (p2, p);
 	      p2->right = p->right;
 	      p2->left = p->left;
 	      if (p->right)
@@ -662,11 +651,11 @@ _asn1_expand_identifier (ASN1_TYPE * node, ASN1_TYPE root)
  * Returns: %ASN1_SUCCESS if creation OK, %ASN1_ELEMENT_NOT_FOUND if
  *   @source_name is not known.
  **/
-asn1_retCode
-asn1_create_element (ASN1_TYPE definitions, const char *source_name,
-		     ASN1_TYPE * element)
+int
+asn1_create_element (asn1_node definitions, const char *source_name,
+		     asn1_node * element)
 {
-  ASN1_TYPE dest_node;
+  asn1_node dest_node;
   int res;
 
   dest_node = _asn1_copy_structure2 (definitions, source_name);
@@ -698,10 +687,10 @@ asn1_create_element (ASN1_TYPE definitions, const char *source_name,
  * from the @name element inside the structure @structure.
  **/
 void
-asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
+asn1_print_structure (FILE * out, asn1_node structure, const char *name,
 		      int mode)
 {
-  ASN1_TYPE p, root;
+  asn1_node p, root;
   int k, indent = 0, len, len2, len3;
 
   if (out == NULL)
@@ -720,7 +709,7 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 	  for (k = 0; k < indent; k++)
 	    fprintf (out, " ");
 	  fprintf (out, "name:");
-	  if (p->name)
+	  if (p->name[0] != 0)
 	    fprintf (out, "%s  ", p->name);
 	  else
 	    fprintf (out, "NULL  ");
@@ -729,15 +718,15 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 	{
 	  switch (type_field (p->type))
 	    {
-	    case TYPE_CONSTANT:
-	    case TYPE_TAG:
-	    case TYPE_SIZE:
+	    case ASN1_ETYPE_CONSTANT:
+	    case ASN1_ETYPE_TAG:
+	    case ASN1_ETYPE_SIZE:
 	      break;
 	    default:
 	      for (k = 0; k < indent; k++)
 		fprintf (out, " ");
 	      fprintf (out, "name:");
-	      if (p->name)
+	      if (p->name[0] != 0)
 		fprintf (out, "%s  ", p->name);
 	      else
 		fprintf (out, "NULL  ");
@@ -746,74 +735,39 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 
       if (mode != ASN1_PRINT_NAME)
 	{
-	  switch (type_field (p->type))
+	  unsigned type = type_field (p->type);
+	  switch (type)
 	    {
-	    case TYPE_CONSTANT:
+	    case ASN1_ETYPE_CONSTANT:
 	      if (mode == ASN1_PRINT_ALL)
 		fprintf (out, "type:CONST");
 	      break;
-	    case TYPE_TAG:
+	    case ASN1_ETYPE_TAG:
 	      if (mode == ASN1_PRINT_ALL)
 		fprintf (out, "type:TAG");
 	      break;
-	    case TYPE_SIZE:
+	    case ASN1_ETYPE_SIZE:
 	      if (mode == ASN1_PRINT_ALL)
 		fprintf (out, "type:SIZE");
 	      break;
-	    case TYPE_DEFAULT:
+	    case ASN1_ETYPE_DEFAULT:
 	      fprintf (out, "type:DEFAULT");
 	      break;
-	    case TYPE_NULL:
-	      fprintf (out, "type:NULL");
-	      break;
-	    case TYPE_IDENTIFIER:
+	    case ASN1_ETYPE_IDENTIFIER:
 	      fprintf (out, "type:IDENTIFIER");
 	      break;
-	    case TYPE_INTEGER:
-	      fprintf (out, "type:INTEGER");
-	      break;
-	    case TYPE_ENUMERATED:
-	      fprintf (out, "type:ENUMERATED");
-	      break;
-	    case TYPE_TIME:
-	      fprintf (out, "type:TIME");
-	      break;
-	    case TYPE_BOOLEAN:
-	      fprintf (out, "type:BOOLEAN");
-	      break;
-	    case TYPE_SEQUENCE:
-	      fprintf (out, "type:SEQUENCE");
-	      break;
-	    case TYPE_BIT_STRING:
-	      fprintf (out, "type:BIT_STR");
-	      break;
-	    case TYPE_OCTET_STRING:
-	      fprintf (out, "type:OCT_STR");
-	      break;
-	    case TYPE_GENERALSTRING:
-	      fprintf (out, "type:GENERALSTRING");
-	      break;
-	    case TYPE_SEQUENCE_OF:
-	      fprintf (out, "type:SEQ_OF");
-	      break;
-	    case TYPE_OBJECT_ID:
-	      fprintf (out, "type:OBJ_ID");
-	      break;
-	    case TYPE_ANY:
+	    case ASN1_ETYPE_ANY:
 	      fprintf (out, "type:ANY");
 	      break;
-	    case TYPE_SET:
-	      fprintf (out, "type:SET");
-	      break;
-	    case TYPE_SET_OF:
-	      fprintf (out, "type:SET_OF");
-	      break;
-	    case TYPE_CHOICE:
+	    case ASN1_ETYPE_CHOICE:
 	      fprintf (out, "type:CHOICE");
 	      break;
-	    case TYPE_DEFINITIONS:
+	    case ASN1_ETYPE_DEFINITIONS:
 	      fprintf (out, "type:DEFINITIONS");
 	      break;
+            CASE_HANDLED_ETYPES:
+	      fprintf (out, "%s", _asn1_tags[type].desc);
+	      break;
 	    default:
 	      break;
 	    }
@@ -823,22 +777,22 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 	{
 	  switch (type_field (p->type))
 	    {
-	    case TYPE_CONSTANT:
+	    case ASN1_ETYPE_CONSTANT:
 	      if (mode == ASN1_PRINT_ALL)
 		if (p->value)
 		  fprintf (out, "  value:%s", p->value);
 	      break;
-	    case TYPE_TAG:
+	    case ASN1_ETYPE_TAG:
 	      if (mode == ASN1_PRINT_ALL)
 		if (p->value)
 		  fprintf (out, "  value:%s", p->value);
 	      break;
-	    case TYPE_SIZE:
+	    case ASN1_ETYPE_SIZE:
 	      if (mode == ASN1_PRINT_ALL)
 		if (p->value)
 		  fprintf (out, "  value:%s", p->value);
 	      break;
-	    case TYPE_DEFAULT:
+	    case ASN1_ETYPE_DEFAULT:
 	      if (p->value)
 		fprintf (out, "  value:%s", p->value);
 	      else if (p->type & CONST_TRUE)
@@ -846,11 +800,11 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 	      else if (p->type & CONST_FALSE)
 		fprintf (out, "  value:FALSE");
 	      break;
-	    case TYPE_IDENTIFIER:
+	    case ASN1_ETYPE_IDENTIFIER:
 	      if (p->value)
 		fprintf (out, "  value:%s", p->value);
 	      break;
-	    case TYPE_INTEGER:
+	    case ASN1_ETYPE_INTEGER:
 	      if (p->value)
 		{
 		  len2 = -1;
@@ -861,7 +815,7 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 		      fprintf (out, "%02x", (p->value)[k + len2]);
 		}
 	      break;
-	    case TYPE_ENUMERATED:
+	    case ASN1_ETYPE_ENUMERATED:
 	      if (p->value)
 		{
 		  len2 = -1;
@@ -872,11 +826,7 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 		      fprintf (out, "%02x", (p->value)[k + len2]);
 		}
 	      break;
-	    case TYPE_TIME:
-	      if (p->value)
-		fprintf (out, "  value:%s", p->value);
-	      break;
-	    case TYPE_BOOLEAN:
+	    case ASN1_ETYPE_BOOLEAN:
 	      if (p->value)
 		{
 		  if (p->value[0] == 'T')
@@ -885,7 +835,7 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 		    fprintf (out, "  value:FALSE");
 		}
 	      break;
-	    case TYPE_BIT_STRING:
+	    case ASN1_ETYPE_BIT_STRING:
 	      if (p->value)
 		{
 		  len2 = -1;
@@ -899,7 +849,35 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 		    }
 		}
 	      break;
-	    case TYPE_OCTET_STRING:
+	    case ASN1_ETYPE_GENERALIZED_TIME:
+	    case ASN1_ETYPE_UTC_TIME:
+	      if (p->value)
+		{
+		  fprintf (out, "  value:");
+                  for (k = 0; k < p->value_len; k++)
+		    fprintf (out, "%c", (p->value)[k]);
+		}
+	      break;
+	    case ASN1_ETYPE_GENERALSTRING:
+	    case ASN1_ETYPE_NUMERIC_STRING:
+	    case ASN1_ETYPE_IA5_STRING:
+	    case ASN1_ETYPE_TELETEX_STRING:
+	    case ASN1_ETYPE_PRINTABLE_STRING:
+	    case ASN1_ETYPE_UNIVERSAL_STRING:
+	    case ASN1_ETYPE_UTF8_STRING:
+	    case ASN1_ETYPE_VISIBLE_STRING:
+	      if (p->value)
+		{
+		  len2 = -1;
+		  len = asn1_get_length_der (p->value, p->value_len, &len2);
+		  fprintf (out, "  value:");
+		  if (len > 0)
+		    for (k = 0; k < len; k++)
+		      fprintf (out, "%c", (p->value)[k + len2]);
+		}
+	      break;
+	    case ASN1_ETYPE_BMP_STRING:
+	    case ASN1_ETYPE_OCTET_STRING:
 	      if (p->value)
 		{
 		  len2 = -1;
@@ -910,22 +888,11 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 		      fprintf (out, "%02x", (p->value)[k + len2]);
 		}
 	      break;
-	    case TYPE_GENERALSTRING:
-	      if (p->value)
-		{
-		  len2 = -1;
-		  len = asn1_get_length_der (p->value, p->value_len, &len2);
-		  fprintf (out, "  value:");
-		  if (len > 0)
-		    for (k = 0; k < len; k++)
-		      fprintf (out, "%02x", (p->value)[k + len2]);
-		}
-	      break;
-	    case TYPE_OBJECT_ID:
+	    case ASN1_ETYPE_OBJECT_ID:
 	      if (p->value)
 		fprintf (out, "  value:%s", p->value);
 	      break;
-	    case TYPE_ANY:
+	    case ASN1_ETYPE_ANY:
 	      if (p->value)
 		{
 		  len3 = -1;
@@ -936,13 +903,13 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 		      fprintf (out, "%02x", (p->value)[k + len3]);
 		}
 	      break;
-	    case TYPE_SET:
-	    case TYPE_SET_OF:
-	    case TYPE_CHOICE:
-	    case TYPE_DEFINITIONS:
-	    case TYPE_SEQUENCE_OF:
-	    case TYPE_SEQUENCE:
-	    case TYPE_NULL:
+	    case ASN1_ETYPE_SET:
+	    case ASN1_ETYPE_SET_OF:
+	    case ASN1_ETYPE_CHOICE:
+	    case ASN1_ETYPE_DEFINITIONS:
+	    case ASN1_ETYPE_SEQUENCE_OF:
+	    case ASN1_ETYPE_SEQUENCE:
+	    case ASN1_ETYPE_NULL:
 	      break;
 	    default:
 	      break;
@@ -1005,9 +972,9 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
 	{
 	  switch (type_field (p->type))
 	    {
-	    case TYPE_CONSTANT:
-	    case TYPE_TAG:
-	    case TYPE_SIZE:
+	    case ASN1_ETYPE_CONSTANT:
+	    case ASN1_ETYPE_TAG:
+	    case ASN1_ETYPE_SIZE:
 	      break;
 	    default:
 	      fprintf (out, "\n");
@@ -1061,10 +1028,10 @@ asn1_print_structure (FILE * out, ASN1_TYPE structure, const char *name,
  * Returns: %ASN1_SUCCESS if successful, %ASN1_ELEMENT_NOT_FOUND if
  *   @name is not known, %ASN1_GENERIC_ERROR if pointer @num is %NULL.
  **/
-asn1_retCode
-asn1_number_of_elements (ASN1_TYPE element, const char *name, int *num)
+int
+asn1_number_of_elements (asn1_node element, const char *name, int *num)
 {
-  ASN1_TYPE node, p;
+  asn1_node node, p;
 
   if (num == NULL)
     return ASN1_GENERIC_ERROR;
@@ -1079,7 +1046,7 @@ asn1_number_of_elements (ASN1_TYPE element, const char *name, int *num)
 
   while (p)
     {
-      if ((p->name) && (p->name[0] == '?'))
+      if (p->name[0] == '?')
 	(*num)++;
       p = p->right;
     }
@@ -1100,15 +1067,15 @@ asn1_number_of_elements (ASN1_TYPE element, const char *name, int *num)
  *   the OID.
  **/
 const char *
-asn1_find_structure_from_oid (ASN1_TYPE definitions, const char *oidValue)
+asn1_find_structure_from_oid (asn1_node definitions, const char *oidValue)
 {
   char definitionsName[ASN1_MAX_NAME_SIZE], name[2 * ASN1_MAX_NAME_SIZE + 1];
   char value[ASN1_MAX_NAME_SIZE];
-  ASN1_TYPE p;
+  asn1_node p;
   int len;
-  asn1_retCode result;
+  int result;
 
-  if ((definitions == ASN1_TYPE_EMPTY) || (oidValue == NULL))
+  if ((definitions == NULL) || (oidValue == NULL))
     return NULL;		/* ASN1_ELEMENT_NOT_FOUND; */
 
 
@@ -1119,7 +1086,7 @@ asn1_find_structure_from_oid (ASN1_TYPE definitions, const char *oidValue)
   p = definitions->down;
   while (p)
     {
-      if ((type_field (p->type) == TYPE_OBJECT_ID) &&
+      if ((type_field (p->type) == ASN1_ETYPE_OBJECT_ID) &&
 	  (p->type & CONST_ASSIGN))
 	{
 	  strcpy (name, definitionsName);
@@ -1145,24 +1112,24 @@ asn1_find_structure_from_oid (ASN1_TYPE definitions, const char *oidValue)
 
 /**
  * asn1_copy_node:
- * @dst: Destination ASN1_TYPE node.
+ * @dst: Destination asn1_node node.
  * @dst_name: Field name in destination node.
- * @src: Source ASN1_TYPE node.
+ * @src: Source asn1_node node.
  * @src_name: Field name in source node.
  *
- * Create a deep copy of a ASN1_TYPE variable.
+ * Create a deep copy of a asn1_node variable.
  *
  * Returns: Return %ASN1_SUCCESS on success.
  **/
-asn1_retCode
-asn1_copy_node (ASN1_TYPE dst, const char *dst_name,
-		ASN1_TYPE src, const char *src_name)
+int
+asn1_copy_node (asn1_node dst, const char *dst_name,
+		asn1_node src, const char *src_name)
 {
 /* FIXME: rewrite using copy_structure().
  * It seems quite hard to do.
  */
   int result;
-  ASN1_TYPE dst_node;
+  asn1_node dst_node;
   void *data = NULL;
   int size = 0;
 
@@ -1170,27 +1137,27 @@ asn1_copy_node (ASN1_TYPE dst, const char *dst_name,
   if (result != ASN1_MEM_ERROR)
     return result;
 
-  data = _asn1_malloc (size);
+  data = malloc (size);
   if (data == NULL)
     return ASN1_MEM_ERROR;
 
   result = asn1_der_coding (src, src_name, data, &size, NULL);
   if (result != ASN1_SUCCESS)
     {
-      _asn1_free (data);
+      free (data);
       return result;
     }
 
   dst_node = asn1_find_node (dst, dst_name);
   if (dst_node == NULL)
     {
-      _asn1_free (data);
+      free (data);
       return ASN1_ELEMENT_NOT_FOUND;
     }
 
   result = asn1_der_decoding (&dst_node, data, size, NULL);
 
-  _asn1_free (data);
+  free (data);
 
   return result;
 }
