@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010, 2011 Free Software Foundation, Inc.
+ * Copyright (C) 2010-2012 Free Software Foundation, Inc.
  *
  * This file is part of LIBTASN1.
  *
@@ -23,13 +23,18 @@
 int
 main (int argc, char *argv[])
 {
-  asn1_retCode ec = 0;
+  int ec = 0;
   const char *errstr;
+  int verbose = 0;
+  
+  if (argc > 1)
+    verbose = 1; 
 
   do
     {
       errstr = asn1_strerror (ec);
-      asn1_perror (ec);
+      if (verbose != 0)
+        asn1_perror (ec);
       ec++;
     }
   while (errstr);
